#region Copyright
/*
 * Copyright 2005-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
#endregion

using System;
using System.Collections;
using System.Reflection;
using Seasar.Framework.Util;

namespace Seasar.Framework.Aop.Impl
{
    /// <summary>
    /// IS2MethodInvocationC^[tFCX̎
    /// </summary>
    public class MethodInvocationImpl : IS2MethodInvocation
    {
        /// <summary>
        /// Ăяo郁\bhCX^X
        /// </summary>
        private readonly object _target;

        /// <summary>
        /// Ăяo郁\bh
        /// </summary>
        private readonly MethodBase _method;

        /// <summary>
        /// \bhInterceptInterceptor̔z
        /// </summary>
        private readonly IMethodInterceptor[] _interceptors;

        /// <summary>
        /// ĂInterceptor̍ċAx
        /// </summary>
        private int _interceptorsIndex = 1;

        /// <summary>
        /// \bḧ
        /// </summary>
        private readonly object[] _arguments;

        /// <summary>
        /// \bhƂ̃NX̃CX^XS2ReiɊւ
        /// </summary>
        private readonly Hashtable _parameters;

        /// <summary>
        /// RXgN^
        /// </summary>
        /// <param name="target">InterceptIuWFNg</param>
        /// <param name="method">Intercept郁\bhMethodBase</param>
        /// <param name="arguments"></param>
        /// <param name="interceptors">\bhInterceptInterceptor</param>
        /// <param name="parameters">Intercept郁\bhƂ̃NX̃CX^XS2ReiɊւ</param>
        public MethodInvocationImpl(object target, MethodBase method,
            object[] arguments, IMethodInterceptor[] interceptors, Hashtable parameters)
        {
            if (target == null)
            {
                throw new NullReferenceException("target");
            }
            if (method == null)
            {
                throw new NullReferenceException("method");
            }
            if (interceptors == null)
            {
                throw new NullReferenceException("interceptors");
            }
            _target = target;
            _method = method;
            _arguments = arguments;
            _interceptors = interceptors;
            _parameters = parameters;
        }

        #region IMethodInvocation o

        /// <summary>
        /// Intercept郁\bhMethod
        /// </summary>
        public MethodBase Method
        {
            get { return _method; }
        }

        /// <summary>
        /// InterceptIuWFNg
        /// </summary>
        public object Target
        {
            get { return _target; }
        }

        /// <summary>
        /// Intercept郁\bḧ
        /// </summary>
        public object[] Arguments
        {
            get { return _arguments; }
        }

        /// <summary>
        /// \bȟĂяo
        /// </summary>
        /// <remarks>
        /// Ƀ`F[ĂInterceptor΁AInterceptorĂяo܂iċAIɌĂяojB
        /// Ƀ`F[ĂInterceptor΁AInterceptĂ郁\bhs܂B
        /// <remarks>
        /// <returns>Interceptꂽ\bh̖߂l</returns>
        public object Proceed()
        {
            while (_interceptorsIndex < _interceptors.Length)
            {
                // Interceptor΁AInterceptorĂяo
                return _interceptors[_interceptorsIndex++].Invoke(this);
            }

            try
            {
                // Interceptꂽ\bhs
                return _method.Invoke(_target, _arguments);
            }
            catch (TargetInvocationException ex)
            {
                // InnerExceptionStackTraceۑ
                ExceptionUtil.SaveStackTraceToRemoteStackTraceString(ex.InnerException);

                // InnerExceptionthrow
                throw ex.InnerException;
            }
        }

        #endregion

        #region IS2MethodInvocation o

        /// <summary>
        /// \bhNX̌^
        /// </summary>
        public Type TargetType
        {
            get { return _target.GetType(); }
        }

        /// <summary>
        /// \bhƂ̃NX̃CX^XS2ReiɊւ
        /// </summary>
        public object GetParameter(string name)
        {
            return _parameters[name];
        }

        #endregion
    }
}
